// $Id: analogWrite.c 28 2012-12-12 01:54:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */

/** include files **/
#include "zbasic_arduino.h"

/** local definitions **/

#if	!defined(cbi)
	#define cbi(reg, bit)			reg &= ~_BV(bit)
#endif
#if	!defined(sbi)
	#define sbi(reg, bit)			reg |= _BV(bit)
#endif

// Define the pins corresponding to timer outputs.
#if defined(OCR0) || defined(OCR0A)
	 #define OC0A_PIN				ZB_BYTE_VAL(zb_pinOC0A)
#endif
#if defined(OCR0B)
	 #define OC0B_PIN				ZB_BYTE_VAL(zb_pinOC0B)
#endif
#if defined(OCR1A)
	 #define OC1A_PIN				ZB_BYTE_VAL(zb_pinOC1A)
#endif
#if defined(OCR1B)
	 #define OC1B_PIN				ZB_BYTE_VAL(zb_pinOC1B)
#endif
#if defined(OCR1C)
	 #define OC1C_PIN				ZB_BYTE_VAL(zb_pinOC1C)
#endif
#if defined(OCR2) || defined(OCR2A)
	 #define OC2A_PIN				ZB_BYTE_VAL(zb_pinOC2A)
#endif
#if defined(OCR2B)
	 #define OC2B_PIN				ZB_BYTE_VAL(zb_pinOC2B)
#endif
#if defined(OCR3A)
	 #define OC3A_PIN				ZB_BYTE_VAL(zb_pinOC3A)
#endif
#if defined(OCR3B)
	 #define OC3B_PIN				ZB_BYTE_VAL(zb_pinOC3B)
#endif
#if defined(OCR3C)
	 #define OC3C_PIN				ZB_BYTE_VAL(zb_pinOC3C)
#endif
#if defined(OCR4A)
	 #define OC4A_PIN				ZB_BYTE_VAL(zb_pinOC4A)
#endif
#if defined(OCR4B)
	 #define OC4B_PIN				ZB_BYTE_VAL(zb_pinOC4B)
#endif
#if defined(OCR4C)
	 #define OC4C_PIN				ZB_BYTE_VAL(zb_pinOC4C)
#endif
#if defined(OCR4D)
	 #define OC4D_PIN				ZB_BYTE_VAL(zb_pinOC4D)
#endif
#if defined(OCR5A)
	 #define OC5A_PIN				ZB_BYTE_VAL(zb_pinOC5A)
#endif
#if defined(OCR5B)
	 #define OC5B_PIN				ZB_BYTE_VAL(zb_pinOC5B)
#endif
#if defined(OCR5C)
	 #define OC5C_PIN				ZB_BYTE_VAL(zb_pinOC5C)
#endif

/** external functions **/

/** external data **/

/** callback functions **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

void
analogWrite(uint8_t pin, int val)
{
	ZB_REQUEST_ENTRY(zbArduinoInit);
	uint8_t portPin = mapDigitalPin(pin); 

	pinMode(pin, 1);		// ensure the pin is an output
	if (val == 0)
		digitalWrite(pin, 0);
	else if (val == 255)
		digitalWrite(pin, 1);
#if defined(TCCR0) && defined(COM00) && !defined(__AVR_ATmega8__) && defined(OC0A_PIN)
	else if ((portPin == OC0A_PIN) && ZB_TIMER_AVAILABLE(0))
	{
		sbi(TCCR0, COM00);
		OCR0 = val;
	}
#endif
#if defined(TCCR0A) && defined(COM0A1) && defined(OC0A_PIN)
	else if ((portPin == OC0A_PIN) && ZB_TIMER_AVAILABLE(0))
	{
		sbi(TCCR0A, COM0A1);
		OCR0A = val;
	}
#endif
#if defined(TCCR0A) && defined(COM0B1) && defined(OC0B_PIN)
	else if ((portPin == OC0B_PIN) &&  ZB_TIMER_AVAILABLE(0))
	{
		sbi(TCCR0A, COM0B1);
		OCR0B = val;
	}
#endif
#if defined(TCCR1A) && defined(COM1A1) && defined(OC1A_PIN)
	else if ((portPin == OC1A_PIN) && ZB_TIMER_AVAILABLE(1))
	{
		sbi(TCCR1A, COM1A1);
		OCR1A = val;
	}
#endif
#if defined(TCCR1A) && defined(COM1B1) && defined(OC1B_PIN)
	else if ((portPin == OC1B_PIN) && ZB_TIMER_AVAILABLE(1))
	{
		sbi(TCCR1A, COM1B1);
		OCR1B = val;
	}
#endif
#if defined(TCCR1A) && defined(COM1C1) && defined(OC1C_PIN)
	else if ((portPin == OC1C_PIN) && ZB_TIMER_AVAILABLE(1))
	{
		sbi(TCCR1A, COM1C1);
		OCR1C = val;
	}
#endif
#if defined(TCCR2) && defined(COM21) && defined(OC2A_PIN)
	else if ((portPin == OC2A_PIN) && ZB_TIMER_AVAILABLE(2))
	{
		sbi(TCCR2, COM21);
		OCR2 = val;
	}
#endif
#if defined(TCCR2A) && defined(COM2A1) && defined(OC2A_PIN)
	else if ((portPin == OC2A_PIN) && ZB_TIMER_AVAILABLE(2))
	{
		sbi(TCCR2A, COM2A1);
		OCR2A = val;
	}
#endif
#if defined(TCCR2A) && defined(COM2B1) && defined(OC2B_PIN)
	else if ((portPin == OC2B_PIN) && ZB_TIMER_AVAILABLE(2))
	{
		sbi(TCCR2A, COM2B1);
		OCR2B = val;
	}
#endif
#if defined(TCCR3A) && defined(COM3A1) && defined(OC3A_PIN)
	else if ((portPin == OC3A_PIN) && ZB_TIMER_AVAILABLE(3))
	{
		sbi(TCCR3A, COM3A1);
		OCR3A = val;
	}
#endif
#if defined(TCCR3A) && defined(COM3B1) && defined(OC3B_PIN)
	else if ((portPin == OC3B_PIN) && ZB_TIMER_AVAILABLE(3))
	{
		sbi(TCCR3A, COM3B1);
		OCR3B = val;
	}
#endif
#if defined(TCCR3A) && defined(COM3C1) && defined(OC3C_PIN)
	else if ((portPin == OC3C_PIN) && ZB_TIMER_AVAILABLE(3))
	{
		sbi(TCCR3A, COM3C1);
		OCR3C = val;
	}
#endif
#if defined(TCCR4A) && defined(COM4A1) && defined(OC4A_PIN)
	else if ((portPin == OC4A_PIN) && ZB_TIMER_AVAILABLE(4))
	{
		sbi(TCCR4A, COM4A1);
  #if defined(COM4A0)		// only used on 32U4
		cbi(TCCR4A, COM4A0);
  #endif
		OCR4A = val;
	}
#endif
#if defined(TCCR4A) && defined(COM4B1) && defined(OC4B_PIN)
	else if ((portPin == OC4B_PIN) && ZB_TIMER_AVAILABLE(4))
	{
		sbi(TCCR4A, COM4B1);
		OCR4B = val;
	}
#endif
#if defined(TCCR4A) && defined(COM4C1) && defined(OC4C_PIN)
	else if ((portPin == OC4C_PIN) && ZB_TIMER_AVAILABLE(4))
	{
		sbi(TCCR4A, COM4C1);
		OCR4C = val;
	}
#endif
#if defined(TCCR4A) && defined(COM4D1) && defined(OC4D_PIN)
	else if ((portPin == OC4D_PIN) && ZB_TIMER_AVAILABLE(4))
	{
		sbi(TCCR4C, COM4D1);
		OCR4D = val;
	}
#endif
#if defined(TCCR5A) && defined(COM5A1) && defined(OC5A_PIN)
	else if ((portPin == OC5A_PIN) && ZB_TIMER_AVAILABLE(5))
	{
		sbi(TCCR5A, COM5A1);
		OCR5A = val;
	}
#endif
#if defined(TCCR5A) && defined(COM5B1) && defined(OC5B_PIN)
	else if ((portPin == OC5B_PIN) && ZB_TIMER_AVAILABLE(5))
	{
		sbi(TCCR5A, COM5B1);
		OCR5B = val;
	}
#endif
#if defined(TCCR5A) && defined(COM5C1) && defined(OC5C_PIN)
	else if ((portPin == OC5C_PIN) && ZB_TIMER_AVAILABLE(5))
	{
		sbi(TCCR5A, COM5C1);
		OCR5C = val;
	}
#endif
	else if (val < 128)
		digitalWrite(pin, 0);
	else
		digitalWrite(pin, 1);
}

/*
 ** killPWM
 *
 * For pins associated with a timer compare output, turn off the PWM output.
 *
 */
void
killPWM(uint8_t pin)
{
	pin = mapDigitalPin(pin); 
	if (!pin)
		;
#if defined(TCCR0) && defined(COM00) && !defined(__AVR_ATmega8__) && defined(OC0A_PIN)
	else if (pin == OC0A_PIN)
		cbi(TCCR0, COM00);
#endif
#if defined(TCCR0A) && defined(COM0A1) && defined(OC0A_PIN)
	else if (pin == OC0A_PIN)
		cbi(TCCR0A, COM0A1);
#endif
#if defined(TCCR0A) && defined(COM0B1) && defined(OC0B_PIN)
	else if (pin == OC0B_PIN)
		cbi(TCCR0A, COM0B1);
#endif
#if defined(TCCR1A) && defined(COM1A1) && defined(OC1A_PIN)
	else if (pin == OC1A_PIN)
		cbi(TCCR1A, COM1A1);
#endif
#if defined(TCCR1A) && defined(COM1B1) && defined(OC1B_PIN)
	else if (pin == OC1B_PIN)
		cbi(TCCR1A, COM1B1);
#endif
#if defined(TCCR1A) && defined(COM1C1) && defined(OC1C_PIN)
	else if (pin == OC1C_PIN)
		cbi(TCCR1A, COM1C1);
#endif
#if defined(TCCR2) && defined(COM21) && defined(OC2A_PIN)
	else if (pin == OC2A_PIN)
		cbi(TCCR2, COM21);
#endif
#if defined(TCCR2A) && defined(COM2A1) && defined(OC2A_PIN)
	else if (pin == OC2A_PIN)
		cbi(TCCR2A, COM2A1);
#endif
#if defined(TCCR2A) && defined(COM2B1) && defined(OC2B_PIN)
	else if (pin == OC2B_PIN)
		cbi(TCCR2A, COM2B1);
#endif
#if defined(TCCR3A) && defined(COM3A1) && defined(OC3A_PIN)
	else if (pin == OC3A_PIN)
		cbi(TCCR3A, COM3A1);
#endif
#if defined(TCCR3A) && defined(COM3B1) && defined(OC3B_PIN)
	else if (pin == OC3B_PIN)
		cbi(TCCR3A, COM3B1);
#endif
#if defined(TCCR3A) && defined(COM3C1) && defined(OC3C_PIN)
	else if (pin == OC3C_PIN)
		cbi(TCCR3A, COM3C1);
#endif
#if defined(TCCR4A) && defined(COM4A1) && defined(OC4A_PIN)
	else if (pin == OC4A_PIN)
		cbi(TCCR4A, COM4A1);
#endif
#if defined(TCCR4A) && defined(COM4B1) && defined(OC4B_PIN)
	else if (pin == OC4B_PIN)
		cbi(TCCR4A, COM4B1);
#endif
#if defined(TCCR4A) && defined(COM4C1) && defined(OC4C_PIN)
	else if (pin == OC4C_PIN)
		cbi(TCCR4A, COM4C1);
#endif
#if defined(TCCR4A) && defined(COM4D1) && defined(OC4D_PIN)
	else if (pin == OC4D_PIN)
		cbi(TCCR4C, COM4D1);
#endif
#if defined(TCCR5A) && defined(COM5A1) && defined(OC5A_PIN)
	else if (pin == OC5A_PIN)
		cbi(TCCR5A, COM5A1);
#endif
#if defined(TCCR5A) && defined(COM5B1) && defined(OC5B_PIN)
	else if (pin == OC5B_PIN)
		cbi(TCCR5A, COM5B1);
#endif
#if defined(TCCR5A) && defined(COM5C1) && defined(OC5C_PIN)
	else if (pin == OC5C_PIN)
		cbi(TCCR5A, COM5C1);
#endif
}

/** interrupt handlers **/

/** private functions **/
