// $Id: hwSerial.cpp 28 2012-12-12 01:54:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */


/** include files **/
#include "zbasic_arduino.h"

/** local definitions **/
#if defined(ZB_HAS_USART)
  #define serialEvent_implemented
#endif
#if defined(USART1_RX_vect) && !defined(ZB_COM1_USES_USART1)
  #define serialEvent1_implemented
#endif
#if defined(USART2_RX_vect)
  #define serialEvent2_implemented
#endif
#if defined(USART3_RX_vect)
  #define serialEvent3_implemented
#endif

/** external functions **/

/** external data **/

/** callback functions **/

/** internal functions **/

/** public data **/
#if defined(ZB_HAS_USART)
	HardwareSerial Serial(1);
  #if defined(UBRR1H) && !defined(ZB_COM1_USES_USART1)
	HardwareSerial Serial1(2);
  #endif
  #if defined(UBRR2H)
	HardwareSerial Serial2(7);
  #endif
  #if defined(UBRR3H)
	HardwareSerial Serial3(8);
  #endif
#endif

/** private data **/

/** class implementations **/

#if defined(ZB_HAS_USART)

HardwareSerial::
HardwareSerial(uint8_t chan)
{
	m_chan = chan;
}

void HardwareSerial::
begin(long baud)
{
	begin(baud, 0xff);
}

void HardwareSerial::
begin(unsigned long baud, uint8_t config)
{
	// get the queue pointers and close the channel
	zbQueue_t *iq = getQPtr();
	zbQueue_t *oq = getQPtr(true);

	// wait for all pending characters to be sent
	while ((zbStatusCom(m_chan) & 0x07) == 0x07)
		;

	// handle the optional configuration value
	if (config != 0xff)
	{
		// prepare the 'flags' and 'stopBits' parameters for zbDefineCom()
		uint8_t flags = ((config >> 1) & 0x03) + 5;
		if (config & 0x20)
			flags |= ((config & 0x30) ^ 0x10);
		uint8_t stopBits = (config & 0x08) ? 2 : 1;

		// configure the UART channel
		zbDefineCom(m_chan, 0, 0, flags, stopBits);
	}

	// re-open the channel with the new parameters and the old queues
	zbOpenCom(m_chan, baud, iq, oq);
}

void HardwareSerial::
end()
{
}

int HardwareSerial::
available(void)
{
	int cnt = 0;
	zbQueue_t *rxQPtr = getQPtr();
	if (rxQPtr != NULL)
		cnt = zbGetQueueCnt(rxQPtr);
	return(cnt);
}

int HardwareSerial::
peek(void)
{
	int c = -1;
	zbQueue_t *rxQPtr = getQPtr();
	if (rxQPtr != NULL)
	{
		uint8_t ch;
		if (zbGetQueueData(rxQPtr, &ch, 1, ZB_QUEUE_PEEK))
			c = ch;
	}
	return(c);
}

int HardwareSerial::
read(void)
{
	int c = -1;
	zbQueue_t *rxQPtr = getQPtr();
	if (rxQPtr != NULL)
		c = zbGetQueueByte(rxQPtr);
	return(c);

}

void HardwareSerial::
flush(void)
{
#if 0
	// this the behavior for Arduino 0023 and earlier
	// clear the receive queue
	zbQueue_t *rxQPtr = getQPtr();
	if (rxQPtr != NULL)
		zbClearQueue(rxQPtr);
#endif

	// this the behavior for Arduino 1.00 and later
	// wait for the transmission to complete
	while (zbStatusComHW(m_chan) & 0x04)
		;
}

size_t HardwareSerial::
write(uint8_t c)
{
	zbQueue_t *txQPtr = getQPtr(true);
	if (txQPtr != NULL)
		zbPutQueueByte(txQPtr, c);
	return(1);
}

zbQueue_t * HardwareSerial::
getQPtr(bool txQ)
{
	zbQueue_t *q = zbGetQPtr(m_chan, static_cast<uint8_t>(txQ));
	return(q);
}

#if defined(serialEvent_implemented)
void serialEvent() __attribute__((weak));
void serialEvent(){}
#endif
#if defined(serialEvent1_implemented)
void serialEvent1() __attribute__((weak));
void serialEvent1(){}
#endif
#if defined(serialEvent2_implemented)
void serialEvent2() __attribute__((weak));
void serialEvent2(){}
#endif
#if defined(serialEvent3_implemented)
void serialEvent3() __attribute__((weak));
void serialEvent3(){}
#endif

void serialEventRun(void)
{
#if defined(serialEvent_implemented)
  if (Serial.available()) serialEvent();
#endif
#if defined(serialEvent1_implemented)
  if (Serial1.available()) serialEvent1();
#endif
#if defined(serialEvent2_implemented)
  if (Serial2.available()) serialEvent2();
#endif
#if defined(serialEvent3_implemented)
  if (Serial3.available()) serialEvent3();
#endif
}

#endif	// defined(ZB_HAS_USART)

/** public functions **/

/** interrupt handlers **/

/** private functions **/
