// $Id: init.c 28 2012-12-12 01:54:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */

/** include files **/
#include "zbasic_arduino.h"

/** local definitions **/

/** external functions **/

/** external data **/

/** callback functions **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/*
 ** zbArduinoInit
 *
 * This function performs initialization of various hardware elements
 * in a manner that is compatible with the Arduino code base and ZBasic.
 *
 */
void zbArduinoInit()
{
	ZB_PROVIDE_ENTRY(zbArduinoInit);

	// prepare Timer0 for Fast PWM
#if defined(TCCR0) && defined(WGM01)
	if (ZB_TIMER_AVAILABLE(0))
		TCCR0 = _BV(WGM01) | _BV(WGM00) | ZB_BYTE_VAL(zb_clkSelPWM0);
#elif defined(TCCR0A) && defined(WGM01)
	if (ZB_TIMER_AVAILABLE(0))
	{
		TCCR0A = _BV(WGM01) | _BV(WGM00);
		TCCR0B = ZB_BYTE_VAL(zb_clkSelPWM0);
	}
#endif  

	// prepare Timer1 for 8-bit Phase/Frequency Correct PWM
#if defined(TCCR1) && defined(WGM10)
	if (ZB_TIMER_AVAILABLE(1))
		TCCR0 = _BV(WGM10) | ZB_BYTE_VAL(zb_clkSelPWM1);
#elif defined(TCCR1A) && defined(WGM10)
	if (ZB_TIMER_AVAILABLE(1))
	{
		TCCR1A = _BV(WGM10);
		TCCR1B = ZB_BYTE_VAL(zb_clkSelPWM1);
	}
#endif  

	// prepare Timer2 for Phase/Frequency Correct PWM
#if defined(TCCR2) && defined(WGM20)
	if (ZB_TIMER_AVAILABLE(2))
		TCCR2 = _BV(WGM20) | ZB_BYTE_VAL(zb_clkSelPWM2);
#elif defined(TCCR2A) && defined(WGM20)
	if (ZB_TIMER_AVAILABLE(2))
	{
		TCCR2A = _BV(WGM20);
		TCCR2B = ZB_BYTE_VAL(zb_clkSelPWM2);
	}
#endif  

	// prepare Timer3 for 8-bit Phase/Frequency Correct PWM
#if defined(TCCR3A) && defined(WGM30)
	if (ZB_TIMER_AVAILABLE(3))
	{
		TCCR3A = _BV(WGM30);
		TCCR3B = ZB_BYTE_VAL(zb_clkSelPWM3);
	}
#endif  

	// prepare Timer4 for 8-bit Phase/Frequency Correct PWM
#if defined(TCCR4A) && defined(TCCR4D)
	if (ZB_TIMER_AVAILABLE(4))
	{
		TCCR4A = _BV(PWM4A) | _BV(PWM4B);
		TCCR4B = ZB_BYTE_VAL(zb_clkSelPWM4);
		TCCR4C = _BV(PWM4D);
		TCCR4D =  _BV(WGM40);
	}
#elif defined(TCCR4A) && defined(WGM40)
	if (ZB_TIMER_AVAILABLE(4))
	{
		TCCR4A = _BV(WGM40);
		TCCR4B = ZB_BYTE_VAL(zb_clkSelPWM4);
	}
#endif  

	// prepare Timer5 for 8-bit Phase/Frequency Correct PWM
#if defined(TCCR5A) && defined(WGM50)
	if (ZB_TIMER_AVAILABLE(5))
	{
		TCCR5A = _BV(WGM50);
		TCCR5B = ZB_BYTE_VAL(zb_clkSelPWM5);
	}
#endif  
}

/** interrupt handlers **/

/** private functions **/
