// $Id: millis_avr.S 28 2012-12-12 01:54:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */

#include <avr/io.h>
#include "zbasic_arduino.h"

#undef SREG
#define SREG							0x3f

// non-initialized data
	.section	.noinit,"aw",@nobits

//---------------------------------------------------------------------------

	.section .text

//---------------------------------------------------------------------------

/*
 ** millis
 *
 * Return the number of milliseconds represented by the current RTC tick value.
 * The conversion from RTC ticks to milliseconds uses two link-time symbols
 * provided by the ZBasic compiler to realize the following equation:
 *
 *    millis = (RTCticks * zb_msMul) / zb_msDiv
 *
 * where the values of zb_msMul and zb_msDiv are in the range 1-255 except that
 * zb_msDiv == 256 is handled as a special case.  The division operation is
 * followed by a rounding step where the result is rounded up if the remainder
 * is greater than or equal to the half of the divisor.
 *
 * On entry:   <nothing>
 *
 * On exit:    r25:22 has the millisecond equivalent
 *
 * C-callable: uint32_t millis(void)
 *
 * Modifies:   r0, r18-25, r30, r31
 *
 */
	.global millis
millis:
#define sreg		r18

	// get the current tick value from the RTC
	ldi		r30, lo8(zb_rtc + 4)
	ldi		r31, hi8(zb_rtc + 4)
	in		sreg, SREG
	cli
	ldd		r22, Z + 0
	ldd		r23, Z + 1
	ldd		r24, Z + 2
	ldd		r25, Z + 3
	out		SREG, sreg

	.global rtcTicks_to_millis
rtcTicks_to_millis:
	// convert ticks to milliseconds using the multiplication and division factors
	ldi		r20, lo8(zb_msMul)
	cpi		r20, 1
	breq	1f
	mul32_8	r21, r25, r24, r23, r22, r20, r19, r18
1:
	ldi		r20, lo8(zb_msDiv)
	cpi		r20, 1
	breq	2f
	div40_8	r21, r25, r24, r23, r22, r20, r19, r18
2:
	ret
#undef sreg

//---------------------------------------------------------------------------

	.end

