// $Id: usDelay_avr.S 28 2012-12-12 01:54:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */

#include <avr/io.h>
#include "zbasic_arduino.h"

// non-initialized data
	.section	.noinit,"aw",@nobits

//---------------------------------------------------------------------------

	.section .text

#if !defined(zero)
	#define zero			r1
#endif

//---------------------------------------------------------------------------

/*
 ** delayMicroseconds
 *
 * Delay for approximately the number of microseconds requested.  Values
 * for several constants used in this routine are provided at link time;
 * which values depend on the operating frequency of the device.  Note
 * that if interrupts are enabled, the delay could be significantly
 * longer than requested.
 *
 * On entry:   r25:24 has the desired microsecond delay
 *
 * On exit:    <nothing>
 *
 * C-callable: void delayMicroseconds(uint16_t us)
 *
 * Modifies:   r25, r24, r23, r22
 *
 */
	.global delayMicroseconds
delayMicroseconds:
#define usHi	r25
#define usLo	r24
#define cntHi	r23
#define cntLo	r22

	// check for a requested delay less than the minimum
	subi	usLo, lo8(zb_usDelay_min)
	sbci	usHi, hi8(zb_usDelay_min)
	brlo	usDelay_done

	// check for a delay larger than 100uS, delay in units of 100uS
	cpi		usLo, lo8(100)
	cpc		usHi, zero
	brlo	delay_10us
delay_100us:
	subi	usLo, lo8(100)
	sbc		usHi, zero
	ldi		cntLo, lo8(zb_100us_loopCount)
	ldi		cntHi, hi8(zb_100us_loopCount)
delay_100us_loop:
	// N.B.: this is a 4-cycle loop
	subi	cntLo, 1
	sbci	cntHi, 0
	brne	delay_100us_loop
	rjmp	delay_100us

	// the remaining delay is less than 100uS, delay in units of 10uS
delay_10us:
	cpi		usLo, lo8(10)
	brlo	delay_1us
	subi	usLo, lo8(10)
	sbc		usHi, zero
	ldi		cntLo, lo8(zb_10us_loopCount)
delay_10us_loop:
	// N.B.: this is a 3-cycle loop
	dec		cntLo
	brne	delay_10us_loop
	rjmp	delay_10us

	// the remaining delay is less than 10uS, delay in units of 1uS
delay_1us:
	// check for a zero loop count (i.e. operating freq. < 3MHz)
	cpi		usHi, lo8(zb_1us_loopCount)
	breq	usDelay_done

	// adjust for the overhead to get here
	subi	usLo, lo8(zb_1us_adjust)
delay_1us_loop1:
	dec		usLo
	brmi	usDelay_done
	ldi		cntLo, lo8(zb_1us_loopCount)
delay_1us_loop2:
	// N.B.: this is a 3-cycle loop
	dec		cntLo
	brne	delay_1us_loop2
	rjmp	delay_1us_loop1

usDelay_done:
	ret
#undef usHi
#undef usLo
#undef cntHi
#undef cntLo

//---------------------------------------------------------------------------

	.end

