// $Id: zbasic_arduino.h 33 2012-12-14 18:10:18Z Don $

/*-----------------------------------------------------------------------------
 *
 *	BSD License
 *  -----------
 *
 *  Copyright (c) 2012 Elba Corp.  All rights reserved.
 *
 *     Elba Corp.
 *     10915 SW Avocet Ct.
 *     Beaverton, OR  97007
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted only for use by licensees of the ZBasic
 *  Compiler for generic AVR devices and only then if all the following
 *  conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the copyright owner nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission of the copyright owner.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------*/

/*
 *
 * The contents of this file were derived, in part, from source files
 * distributed as part of the Arduino project.  The goal is to provide
 * an interface between Arduino functionality and ZBasic infrastructure
 * thus allowing Arduino code to be used in ZBasic applications.
 *
 */

#ifndef __ZBASIC_ARDUINO_H__
#define __ZBASIC_ARDUINO_H__

#include "zbasic.h"
#if !defined(__ASSEMBLER__)
	#include "Arduino.h"
	#include "wiring_private.h"
#endif

#if defined(UBRRH) || defined(UBRR0H) || defined(UBRR1H) || defined(UBRR2H) || defined(UBRR3H)
	#define ZB_HAS_USART
#endif

#if defined(__AVR_ATmega16__) || defined(__AVR_ATmega16A__) || \
		defined(__AVR_ATmega32__) || defined(__AVR_ATmega32A__) || \
		defined(__AVR_ATmega8__) || defined(__AVR_ATmega8A__)
	#define EICRA						MCUCR
	#define EIFR						GIFR
	#define EIMSK						GICR
#endif

#if defined(__AVR_ATmega16U4__) || defined(__AVR_ATmega32U4__)
	#define USE_ANALOG_PIN_MAP
#endif

#if defined(INT7_vect)
	#define NUM_EXT_INT					8
#elif defined(INT6_vect)
	#define NUM_EXT_INT					7
#elif defined(INT5_vect)
	#define NUM_EXT_INT					6
#elif defined(INT4_vect)
	#define NUM_EXT_INT					5
#elif defined(INT3_vect)
	#define NUM_EXT_INT					4
#elif defined(INT2_vect)
	#define NUM_EXT_INT					3
#elif defined(INT1_vect)
	#define NUM_EXT_INT					2
#else
	#define NUM_EXT_INT					1
#endif

#if !defined(__ASSEMBLER__)
//---------------------------------------------------------------------------
#ifdef __cplusplus
extern "C" {
#endif


// map F_CPU to the compiler-supplied operating frequency
#undef F_CPU
#define F_CPU							ZB_DWORD_VAL(zb_cpuFreq)

// map pin/port macros to ZBasic equivalents
#undef digitalPinToPort
#define digitalPinToPort(P)				(ZB_PORT_IDX(mapDigitalPin(P)) + 1)
#undef digitalPinToBitMask
#define digitalPinToBitMask(P)			zbGetPortMask(mapDigitalPin(P))
#undef portInputRegister
#define portInputRegister(P)			((volatile uint8_t *)zbGetPortAddrByIdx(P - 1, 0))
#undef portModeRegister
#define portModeRegister(P)				((volatile uint8_t *)zbGetPortAddrByIdx(P - 1, 1))
#undef portOutputRegister
#define portOutputRegister(P)			((volatile uint8_t *)zbGetPortAddrByIdx(P - 1, 2))

#define ZB_TIMER_AVAILABLE(n)			(ZB_BYTE_VAL(zb_timerAvail) & _BV(n))

// SPI and I2C pins
#define MOSI							ZB_BYTE_VAL(zb_pinMOSI)
#define MISO							ZB_BYTE_VAL(zb_pinMISO)
#define SCK								ZB_BYTE_VAL(zb_pinSCK)
#define SS								ZB_BYTE_VAL(zb_pinSS)
#define SDA								ZB_BYTE_VAL(zb_pinSDA)
#define SCL								ZB_BYTE_VAL(zb_pinSCL)

#if defined(USE_ANALOG_PIN_MAP)
extern const uint8_t analogPinMap[] PROGMEM;
#endif
extern const uint8_t digitalPinMap[] PROGMEM;
extern const uint8_t intNumMap[] PROGMEM;
extern volatile voidFuncPtr intFunc[NUM_EXT_INT];

void killPWM(uint8_t pin);
uint8_t mapDigitalPin(uint8_t pin);
void attachInt0(void (*func)(void), uint8_t mode);
void attachInt1(void (*func)(void), uint8_t mode);
void attachInt2(void (*func)(void), uint8_t mode);
void attachInt3(void (*func)(void), uint8_t mode);
void attachInt4(void (*func)(void), uint8_t mode);
void attachInt5(void (*func)(void), uint8_t mode);
void attachInt6(void (*func)(void), uint8_t mode);
void attachInt7(void (*func)(void), uint8_t mode);

#if defined(__cplusplus)
}

  #if defined(ZB_HAS_USART)
#include "Stream.h"

class HardwareSerial : public Stream
{
public:
	HardwareSerial(uint8_t chan);
	void begin(long baud);
    void begin(unsigned long baud, uint8_t config);
	void end();
	virtual int available(void);
	virtual int peek(void);
	virtual int read(void);
	virtual void flush(void);
	virtual size_t write(uint8_t c);
#if !defined(ZBASIC_IMPORT)
    inline size_t write(unsigned long n) { return write((uint8_t)n); }
    inline size_t write(long n) { return write((uint8_t)n); }
    inline size_t write(unsigned int n) { return write((uint8_t)n); }
    inline size_t write(int n) { return write((uint8_t)n); }
#endif
	using Print::write;
    operator bool() { return true; }

private:
	// prevent copy construction and assignment
	HardwareSerial(const HardwareSerial&);
	HardwareSerial& operator=(const HardwareSerial&);
	
	zbQueue_t *getQPtr(bool txQ = false);
	uint8_t m_chan;
};

// Define config for Serial.begin(baud, config);
#define SERIAL_5N1 0x00
#define SERIAL_6N1 0x02
#define SERIAL_7N1 0x04
#define SERIAL_8N1 0x06
#define SERIAL_5N2 0x08
#define SERIAL_6N2 0x0A
#define SERIAL_7N2 0x0C
#define SERIAL_8N2 0x0E
#define SERIAL_5E1 0x20
#define SERIAL_6E1 0x22
#define SERIAL_7E1 0x24
#define SERIAL_8E1 0x26
#define SERIAL_5E2 0x28
#define SERIAL_6E2 0x2A
#define SERIAL_7E2 0x2C
#define SERIAL_8E2 0x2E
#define SERIAL_5O1 0x30
#define SERIAL_6O1 0x32
#define SERIAL_7O1 0x34
#define SERIAL_8O1 0x36
#define SERIAL_5O2 0x38
#define SERIAL_6O2 0x3A
#define SERIAL_7O2 0x3C
#define SERIAL_8O2 0x3E

  #if defined(__AVR_ATmega8U2__) || defined(__AVR_ATmega16U2__) || defined(__AVR_ATmega32U2__) || \
		defined(__AVR_ATmega16U4__) || defined(__AVR_ATmega32U4__) || \
		defined(__AVR_AT90USB82__) || defined(__AVR_AT90USB162__) || \
		defined(__AVR_AT90USB646__) || defined(__AVR_AT90USB1286__) || \
		defined(__AVR_AT90USB647__) || defined(__AVR_AT90USB1287__)
	// these devices have no USART0
	#define ZB_COM1_USES_USART1
  #endif

  #if defined(UBRRH) || defined(UBRR0H)
	extern HardwareSerial Serial;
  #endif
  #if defined(UBRR1H) && !defined(ZB_COM1_USES_USART1)
	extern HardwareSerial Serial1;
  #endif
  #if defined(UBRR2H)
	extern HardwareSerial Serial2;
  #endif
  #if defined(UBRR3H)
	extern HardwareSerial Serial3;
  #endif
  #endif	// defined(ZB_HAS_USART)
#endif	// defined(__cplusplus)

#if defined(USBCON)
	// temporary placeholders
	#define USB_PID			0x8036	
	#define USB_VID			0x2341

	#define TX_RX_LED_INIT	DDRD |= (1<<5), DDRB |= (1<<0)
	#define TXLED0			PORTD |= (1<<5)
	#define TXLED1			PORTD &= ~(1<<5)
	#define RXLED0			PORTB |= (1<<0)
	#define RXLED1			PORTB &= ~(1<<0)
#endif

static const uint8_t zbChanSPI = 1;			// define the ZBasic SPI channel to use for Ethernet, WiFi, etc
static const uint8_t zbChanSPI_SD = 2;		// define the ZBasic SPI channel to use for SD cards

//---------------------------------------------------------------------------
#endif	// !defined(__ASSEMBLER__)

//---------------------------------------------------------------------------
#if defined(__ASSEMBLER__)

/*
 * Macro to multiply a 4-byte value by a 1-byte value yielding a 5-byte result.
 *
 * On entry:   _A3.._A0 contains the 4-byte multiplicand
 *             _B0      contains the 1-byte multiplier
 *
 * On exit:    _A4.._A0 contains the 5-byte product
 *
 * Modifies:   _A4.._A0, r0, _tmp, _cnt
 *
 */
// Uncomment the line below to use the AVR's hardware multiplier (if available).
// Using the hardware multiplier yields faster code at the expense of larger code size.
#define USE_HW_MUL

	.macro	mul32_8	_A4, _A3, _A2, _A1, _A0, _B0, _tmp, _cnt
	// special case: multiplying by 1
  #if defined(USE_HW_MUL) && defined(__AVR_HAVE_MUL__)
	mul		\_A0, \_B0
	mov		\_A0, r0
	mov		\_tmp, r1

	mul		\_A1, \_B0
	mov		\_A1, \_tmp
	add		\_A1, r0
	mov		\_tmp, r1
	clr		r1
	adc		\_tmp, r1

	mul		\_A2, \_B0
	mov		\_A2, \_tmp
	add		\_A2, r0
	mov		\_tmp, r1
	clr		r1
	adc		\_tmp, r1

	mul		\_A3, \_B0
	mov		\_A3, \_tmp
	add		\_A3, r0
	mov		\_A4, r1
	clr		r1
	adc		\_A4, r1
  #else
	// multiply using add/shift
	clr		\_A4
	clr		\_tmp
	ldi		\_cnt, 40 + 1
	rjmp	2f
0:
	brcc	1f
	add		\_tmp, \_B0
1:
	ror		\_tmp
2:
	ror		\_A4
	ror		\_A3
	ror		\_A2
	ror		\_A1
	ror		\_A0
	dec		\_cnt
	brne	0b
  #endif
3:
	.endm

//---------------------------------------------------------------------------

/*
 * Macro to divide a 5-byte value by a 1-byte value yielding a 4-byte result.
 * Note that the value 0 for the divisor is handled as dividing by 256.
 *
 * On entry:   _A4.._A0 contains the 5-byte dividend
 *             _B0      contains the 1-byte divisor
 *
 * On exit:    _A3.._A0 contains the rounded 4-byte quotient
 *
 * C-callable: no
 *
 * Modifies:   _A4.._A0, r0, _tmp, _cnt
 *
 */
	.macro	div40_8	_A4, _A3, _A2, _A1, _A0, _B0, _tmp, _cnt
	clr		\_tmp
	ldi		\_cnt, 40

	// special case: dividing by 256
	tst		\_B0
	brne	0f
	mov		\_tmp, \_A0
	mov		\_A0, \_A1
	mov		\_A1, \_A2
	mov		\_A2, \_A3
	mov		\_A3, \_A4
	ldi		\_B0, 0x80
	rjmp	3f

0:
	// proceed with the division
	lsl		\_A0
	rol		\_A1
	rol		\_A2
	rol		\_A3
	rol		\_A4
	rol		\_tmp
	brcs	1f
	cp		\_tmp, \_B0
	brcs	2f
1:
	sub		\_tmp, \_B0
	ori		\_A0, 1
2:
	dec		\_cnt
	brne	0b

	// round up if the remainder is >= half the divisor
	lsr		\_B0
3:
	cp		\_tmp, \_B0
	brlo	4f
	subi	\_A0, 0xff
	sbci	\_A1, 0xff
	sbci	\_A2, 0xff
	sbci	\_A3, 0xff
4:
	.endm
#endif	// defined(__ASSEMBLER__)

//---------------------------------------------------------------------------

#endif	// __ZBASIC_ARDUINO_H__
